<?php

namespace Drupal\menu_to_taxonomy_assign;

use Drupal\Core\Field\EntityReferenceFieldItemList;
use Drupal\node\NodeInterface;
use Drupal\taxonomy\Entity\Term;

/**
 * Defines a item list class for entity reference fields.
 */
class AutoAssignedTermReferenceItemList extends EntityReferenceFieldItemList implements AutoAssignedTermReferenceItemListInterface {

  /**
   * {@inheritdoc}
   */
  public function updateAutoAssigned() {
    $values = parent::getValue();
    $entity = $this->getEntity();
    $hasNewValues = FALSE;
    if ($entity->getEntityTypeId() == 'node') {
      /** @var \Drupal\menu_to_taxonomy\MenuToTaxonomySyncRepositoryInterface $service */
      $service = \Drupal::service('menu_to_taxonomy.sync_repository');
      $settings = $this->getSettings();
      if (isset($settings['handler_settings']['target_bundles'])) {
        $vids = array_values($settings['handler_settings']['target_bundles']);
        foreach ($vids as $vid) {
          $link = $this->getLinkFromNode($entity);
          if (empty($link)) {
            continue;
          }
          $id = $link->getMenuPluginId();
          $tid = $service->getTid($id, $vid);
          if (empty($tid) || in_array(
                $tid, array_map(
                    function ($x) {
                            return $x['target_id'];
                    }, $values
                )
            )
            ) {
            continue;
          }
          $term = Term::load($tid);
          if (empty($term)) {
            continue;
          }
          if (!$hasNewValues) {
            $hasNewValues = TRUE;
          }
          $values[] = $tid;
        }
      }
    }
    if ($hasNewValues) {
      $this->setValue($values, FALSE);
    }
    return $hasNewValues;
  }

  protected function getLinkFromNode(NodeInterface $node) {
    if (!$node->hasField('field_menulink')) {
      return FALSE;
    }
    /** @var \Drupal\menu_link\Plugin\Field\FieldType\MenuLinkItem $link */
    $link = $node->get('field_menulink')[0];
    if (empty($link)) {
      return FALSE;
    }
    return $link;
  }

}
